/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.BaseTransientBottomBar
import com.google.android.material.snackbar.Snackbar
import com.kakao.sdk.auth.AuthApiClient
import com.kakao.sdk.partner.auth.model.AgeAuthLevel
import com.kakao.sdk.partner.auth.verifyAge
import com.kakao.sdk.partner.talk.*
import com.kakao.sdk.partner.talk.model.*
import com.kakao.sdk.partner.user.*
import com.kakao.sdk.sample.common.OpenFragment
import com.kakao.sdk.sample.common.internal.ApiAdapter
import com.kakao.sdk.sample.common.internal.FriendsActivity
import com.kakao.sdk.sample.common.internal.Log
import com.kakao.sdk.sample.common.internal.PickerItem
import com.kakao.sdk.talk.TalkApiClient
import com.kakao.sdk.talk.model.FriendOrder
import com.kakao.sdk.talk.model.Friends
import com.kakao.sdk.talk.model.Order
import com.kakao.sdk.user.UserApiClient

/**
 * A simple [Fragment] subclass.
 */
class PartnerFragment : Fragment() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let { templateIds = it }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_partner, container, false)
    }

    private fun showSnackbar(any: Any) {
        val snackbar = Snackbar.make(view!!, any.toString(), BaseTransientBottomBar.LENGTH_LONG)
        val layout = snackbar.view as Snackbar.SnackbarLayout
        layout.minimumHeight = 100
        snackbar.show()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewManager = LinearLayoutManager(context!!)
        viewAdapter = ApiAdapter(
            listOf(
                ApiAdapter.Item.Header("Auth API"),
                ApiAdapter.Item.ApiItem("verifyAge()") {
                    AuthApiClient.instance.verifyAge(context!!) { error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        } else {
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("verifyAge(1차 인증)") {
                    AuthApiClient.instance.verifyAge(
                        context!!,
                        authLevel = AgeAuthLevel.AUTH_LEVEL1
                    ) { error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        } else {
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("verifyAge(2차 인증)") {
                    AuthApiClient.instance.verifyAge(
                        context!!,
                        authLevel = AgeAuthLevel.AUTH_LEVEL2
                    ) { error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        } else {
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }
                    }
                },
                ApiAdapter.Item.Header("User API"),
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - empty") {
                    UserApiClient.instance.loginWithKakaoAccount(
                        context!!,
                        mapOf()
                    ) { token, error ->
                        if (error != null) {
                            Log.e(TAG, "로그인 실패", error)
                        } else if (token != null) {
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - login '@kakaocorp.com'") {
                    UserApiClient.instance.loginWithKakaoAccount(
                        context!!,
                        mapOf("view" to "login", "email" to "@kakaocorp.com")
                    ) { token, error ->
                        if (error != null) {
                            Log.e(TAG, "로그인 실패", error)
                        } else if (token != null) {
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - signup") {
                    UserApiClient.instance.loginWithKakaoAccount(
                        context!!,
                        mapOf("view" to "signup")
                    ) { token, error ->
                        if (error != null) {
                            Log.e(TAG, "로그인 실패", error)
                        } else if (token != null) {
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("meForPartner()") {
                    UserApiClient.instance.meForPartner { user, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("$user")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("signupForPartner()") {
                    UserApiClient.instance.signupForPartner { error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("signup 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("ageAuthInfo()") {
                    UserApiClient.instance.ageAuthInfo { info, error ->
                        if (error != null) {
                            Log.e(TAG, "연령 인증 조회 실패", error)
                        } else {
                            Log.i(TAG, info.toString())
                            showSnackbar(info.toString())
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("ageAuthInfo(account_ci)") {
                    UserApiClient.instance.ageAuthInfo(propertyKeys = listOf("account_ci")) { info, e ->
                        if (e != null) {
                            Log.e(TAG, "연령 인증 조회 실패", e)
                        } else {
                            Log.i(TAG, info.toString())
                            showSnackbar(info.toString())
                            if (info?.ci != null) {
                                Log.i(TAG, "ci 획득  ${info.ci}")
                            } else if (info?.ciNeedsAgreement == false) {
                                Log.e(TAG, "사용자 계정에 ci 없음. 꼭 필요하다면 동의항목 설정에서 수집 기능을 활성화 해보세요.")
                            } else if (info?.ciNeedsAgreement == true) {

                                val scopes = listOf("account_ci")
                                UserApiClient.instance.loginWithNewScopes(
                                    context!!,
                                    scopes
                                ) { _, err ->
                                    if (err != null) {
                                        Log.e(TAG, "ci 제공 동의 실패", err)
                                    } else {
                                        UserApiClient.instance.ageAuthInfo(
                                            12,
                                            listOf("account_ci")
                                        ) { info, error ->
                                            Log.i(TAG, info.toString())
                                            showSnackbar(info.toString())

                                            if (error != null) {
                                                Log.e(TAG, "연령 인증 조회 실패", error)
                                            } else if (info?.ci != null) {
                                                Log.i(TAG, "ci 획득  ${info.ci}")
                                                showSnackbar("ci 획득  ${info.ci}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("upgradeScopes()") {
                    val scopes = listOf("birthday")
                    UserApiClient.instance.upgradeScopes(scopes) { info, error ->
                        if (error != null) {
                            Log.e(TAG, "Upgrade Scopes 실패", error)
                        } else {
                            Log.i(TAG, info.toString())
                            showSnackbar(info.toString())
                        }
                    }
                },
                ApiAdapter.Item.Header("Friend Picker API"),
                ApiAdapter.Item.ApiItem("show friend picker") {
                    Intent(context, FriendPickerSettingActivity::class.java).run {
                        startActivity(this)
                    }
                },
                ApiAdapter.Item.ApiItem("show chat picker") {
                    Intent(context, ChatPickerSettingActivity::class.java).run {
                        startActivity(this)
                    }
                },
                ApiAdapter.Item.ApiItem("show tab picker") {
                    Intent(context, TabPickerSettingActivity::class.java).run {
                        startActivity(this)
                    }
                },
                ApiAdapter.Item.Header("Talk API"),
                ApiAdapter.Item.ApiItem("friendsForPartner()") {
                    TalkApiClient.instance.friendsForPartner { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(friends!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("friendsForPartner(context:) - reculsive") {
                    var nextFriendsContext = PartnerFriendsContext(
                        offset = 0,
                        limit = 10,
                        order = Order.DESC,
                        countryCodes = listOf("kr", "us"),
                        friendType = FriendType.KAKAO_TALK
                    )

                    recursiveAppFriendsCompletion =
                        recursiveAppFriendsCompletion@{ nextFriends, err ->
                            if (err == null) {
                                if (nextFriends != null) {
                                    try {
                                        nextFriends.afterUrl?.let {
                                            nextFriendsContext = PartnerFriendsContext(url = it)
                                        } ?: return@recursiveAppFriendsCompletion

                                    } catch (e: IllegalArgumentException) {
                                        return@recursiveAppFriendsCompletion
                                    }
                                }

                                TalkApiClient.instance.friendsForPartner(context = nextFriendsContext) { friends, error ->
                                    if (error != null) {
                                        Log.e(TAG, "카카오톡 친구 목록 받기 실패", error)
                                    } else if (friends != null) {
                                        Log.i(
                                            TAG,
                                            "카카오톡 친구 목록 받기 성공 \n${friends.elements?.joinToString("\n")}"
                                        )
                                        recursiveAppFriendsCompletion?.let { it(friends, null) }
                                    }
                                }
                            }
                        }

                    recursiveAppFriendsCompletion?.let { it(null, null) }
                },
                ApiAdapter.Item.ApiItem("friendsForPartner(context:) - PartnerFriendsContext") {
                    TalkApiClient.instance.friendsForPartner(
                        context = PartnerFriendsContext(
                            offset = 0,
                            limit = 3,
                            order = Order.DESC,
                            friendOrder = FriendOrder.NICKNAME,
                            friendFilter = FriendFilter.REGISTERED,
                            friendType = FriendType.KAKAO_TALK,
                            countryCodes = listOf("kr", "us")
                        )
                    ) { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(friends!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("chatList()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(chats!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("chatMembers()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (chats != null) {

                            chats.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("채팅방이 없습니다.")
                                } else {

                                    val chatId = it.first().id

                                    TalkApiClient.instance.chatMembers(chatId) { chatMembers, error ->
                                        if (error != null) {
                                            showSnackbar(error)
                                        } else {
                                            showSnackbar(chatMembers!!)
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendCustomMessageForPartner()") {
                    TalkApiClient.instance.friendsForPartner { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            friends.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 친구가 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        context!!,
                                        it.map { friend ->
                                            PickerItem(
                                                friend.uuid,
                                                friend.profileNickname ?: "",
                                                friend.profileThumbnailImage
                                            )
                                        }
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val templateId = templateIds.getLong("customMessage")

                                        TalkApiClient.instance.sendCustomMessageForPartner(
                                            selectedItems,
                                            templateId
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendScrapMessageForPartner()") {
                    TalkApiClient.instance.friendsForPartner { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            friends.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 친구가 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        context!!,
                                        it.map { friend ->
                                            PickerItem(
                                                friend.uuid,
                                                friend.profileNickname ?: "",
                                                friend.profileThumbnailImage
                                            )
                                        }
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val url = "https://developers.kakao.com"

                                        TalkApiClient.instance.sendScrapMessageForPartner(
                                            selectedItems,
                                            url
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessageForPartner()") {
                    TalkApiClient.instance.friendsForPartner { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            friends.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 친구가 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        context!!,
                                        it.map { friend ->
                                            PickerItem(
                                                friend.uuid,
                                                friend.profileNickname ?: "",
                                                friend.profileThumbnailImage
                                            )
                                        }
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val template = OpenFragment.defaultLocation

                                        TalkApiClient.instance.sendDefaultMessageForPartner(
                                            selectedItems,
                                            template
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendCustomMessageToChats()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (chats != null) {
                            chats.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 채팅방이 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        context!!,
                                        it.map(PickerItem.Companion::fromChat)
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val templateId = templateIds.getLong("customMessage")

                                        TalkApiClient.instance.sendCustomMessageToChats(
                                            selectedItems.map { item -> item.toLong() },
                                            templateId
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessageToChats()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (chats != null) {
                            chats.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 채팅방이 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        context!!,
                                        it.map(PickerItem.Companion::fromChat)
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val template = OpenFragment.defaultFeed

                                        TalkApiClient.instance.sendDefaultMessageToChats(
                                            selectedItems.map { item -> item.toLong() },
                                            template
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }

                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendScrapMessageToChats()") {
                    TalkApiClient.instance.chatList { chats, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (chats != null) {
                            chats.elements?.let {
                                if (it.isEmpty()) {
                                    showSnackbar("메시지 보낼 채팅방이 없습니다.")
                                } else {
                                    FriendsActivity.startForResult(
                                        context!!,
                                        it.map(PickerItem.Companion::fromChat)
                                    ) { selectedItems ->
                                        if (selectedItems.isEmpty()) return@startForResult

                                        val url = "https://developers.kakao.com"

                                        TalkApiClient.instance.sendScrapMessageToChats(
                                            selectedItems.map { it.toLong() },
                                            url
                                        ) { result, error ->
                                            if (error != null) {
                                                showSnackbar(error)
                                            } else {
                                                showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverIds}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                })
        )

        view.findViewById<RecyclerView>(R.id.recyclerView).apply {
            setHasFixedSize(true)
            addItemDecoration(
                DividerItemDecoration(context, DividerItemDecoration.VERTICAL)
            )
            layoutManager = viewManager
            adapter = viewAdapter
        }
    }

    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: RecyclerView.LayoutManager

    private lateinit var templateIds: Bundle


    private var recursiveAppFriendsCompletion: ((Friends<PartnerFriend>?, Error?) -> Unit)? = null

    companion object {
        const val TAG = "KakaoSDKSample"

        fun newInstance(templateIds: Bundle): PartnerFragment =
            PartnerFragment().apply {
                arguments = templateIds
            }
    }
}

fun PickerItem.Companion.fromPartnerFriend(friend: PartnerFriend) =
    PickerItem(friend.uuid, friend.profileNickname ?: "", friend.profileThumbnailImage)

fun PickerItem.Companion.fromChat(chat: Chat) =
    PickerItem(
        chat.id.toString(),
        chat.title ?: "",
        chat.displayMemberImages?.let { if (it.isNotEmpty()) it[0] else chat.imageUrl }
    )